#!/bin/bash

# SUpload (Screenshot-Upload)
VERSION="1.0"

## Colours
R='\033[0;31m'
NC='\033[0m'
G='\033[0;32m'

## Default  Configs
## WARNING: Do not change these files here, change them in the config file

SFW=false
DEBUG=false
SSHOT="maim"
FILE_FORMAT="date +%F-%T.png"
FILE_DIR="${HOME}/Pictures/screenshots"
BROWSER=false
CLIPBOARD=true
NOTIFICATION=true
LOG=true
LOG_FILE="${HOME}/.supload.log"
CURL_OPTIONS="--progress-bar"

## Read from user config if exists, else read from system-wide (just use inbuilt defaults if none)
### User config takes priority over system-wide 
CONFIG=$HOME/.config/supload/settings.conf 
if [ -f "${CONFIG}" ] ; then
  source "${CONFIG}"
elif [ -f "/etc/supload.conf" ] ; then
  source "/etc/supload.conf"
fi

## Check dependencies (in case of a bad installation)
type jq >/dev/null 2>&1 || { echo -e >&2 "${R}ERROR:${NC} I require jq but it's not installed.  Aborting."; exit 1; }
type curl >/dev/null 2>&1 || { echo -e >&2 "${R}ERROR:${NC} I require curl but it's not installed.  Aborting."; exit 1; }

usage() {
cat <<EOF
Usage: supload [OPTIONS] [FILE]

Options:

  -f      Safe for work mode, (uses volkor.me)
  -h      Show this menu
  -o      Automatically open in browser
  -s      Screenshot mode (maim Selection)
  -V      Verbose
  -v      Shows current version of SUpload

Examples: https://git.thecum.zone/Volkor/SUpload

EOF
exit 0
}

## Actual file uploading
upload_file() {
  if [[ "$SFW" = "true" ]] ; then
    BASE_URL="https://volkor.me/"
  else
    BASE_URL="https://thecum.zone/"
  fi
  $DEBUG && echo -e "${R}DEBUG:${NC} BASE_URL: $BASE_URL"

  ## Upload to server
  echo "Uploading File: ("$1")"
	response=$(curl $CURL_OPTIONS -F file=@"$1" "$BASE_URL") || { echo -e >&2 "${R}ERROR:${NC} Curl upload failure. (Invalid file, or bad internet connection)"; exit 1; }
  $DEBUG && echo -e "${R}DEBUG:${NC} RESPONSE: $response"
  
  url=$(echo $response | jq -r '.["url"]')
  deletionlink=$(echo $response | jq -r '.["deletionurl"]')
  $DEBUG && echo -e "${R}DEBUG:${NC} URL: $url"

  ## Browser, clipboard and logging.
  if [ "$BROWSER" = "true" ] ; then
    echo "Opening browser"
    hash xdg-open && xdg-open $url &>/dev/null || echo "${R}ERROR:${NC} xdg-open was not found, or failed to run."
  fi

  if [ "$CLIPBOARD" = "true" ] && [ -x "$(command -v xclip)" ]; then
    echo ${url} | tr -d '\n' | xclip -selection clipboard ; echo "Copied URL to clipboard";
  fi
$DEBUG && echo -e "${R}DEBUG:${NC} BROWSER: $BROWSER"
$DEBUG && echo -e "${R}DEBUG:${NC} CLIPBOARD: $CLIPBOARD"

  if [ -f "${LOG_FILE}" ]; then
    if [ "$LOG" = "true" ]; then
      echo -n $(date +"[%Y-%m-%d %H:%M:%S]") >> $LOG_FILE && echo " Upload Response: $response" >> $LOG_FILE
    fi
  else
    touch $LOG_FILE
  fi
  $DEBUG && echo -e "${R}DEBUG:${NC} LOG: $LOG"
  $DEBUG && echo -e "${R}DEBUG:${NC} LOG_FILE: $LOG_FILE"
  upload_finished
}

upload_finished() {
  echo "URL: "${url}
  echo "Deletion URL: "${deletionlink}
  if [ "$NOTIFICATION" = "true" ] ; then
    notify-send "File uploaded." "URL: "${url} & exit 0
  fi
  exit 0
}

while getopts "fhosvV" opt; do
  case $opt in
    f)
	    SFW=true
      ;;
    h)
      usage
      ;;
    o)
	    BROWSER=true
      ;;
    s)
      SCREENSHOT=true
      ;;
    V)
      DEBUG=true
      ;;
    v)
      echo "SUpload Version: "$VERSION
      exit 0
      ;;
  esac
done

FILE="${@:$OPTIND}"
$DEBUG && echo -e "${R}DEBUG:${NC} OPTIONS: $@"

if [ "$SCREENSHOT" = "true" ] ; then
  echo "Click and hold to select screenshot area or click for window."
  SCREEN_PATH="$FILE_DIR/`$FILE_FORMAT`"
  $DEBUG && echo -e "${R}DEBUG:${NC} SCREEN_PATH: $SCREEN_PATH"

  case $SSHOT in
    maim)
      SCREEN_PROG="maim -s -o" ;;
    scrot)
      SCREEN_PROG="scrot -s" ;;
    *)
      echo -e "${R}ERROR:${NC} Invalid SSHOT in config, must be scrot or maim."
      exit 1
      ;;
  esac
  $DEBUG && echo -e "${R}DEBUG:${NC} SSHOT: $SSHOT"

  $($SCREEN_PROG ${SCREEN_PATH}) #> /dev/null 2>&1
  if [ "${?}" != "0" ]; then
    echo -e "${R}ERROR:${NC} Failed to take screenshot (User exited maim)"
    exit 1
  fi
  upload_file ${SCREEN_PATH}
fi

if [ "$1" = "" ] ; then
  echo -e "${R}ERROR:${NC} You must supply either STDIN (-) or a file!"
  exit 1
fi

upload_file "$FILE"